import logging
from typing import List, Tuple

from dateutil import parser
from openpyxl.worksheet.worksheet import Worksheet
from pydantic import ValidationError

from vocexcel import models
from vocexcel.utils import ConversionError, split_and_tidy_to_strings


# Checking how many colon's in string
def only_one_colon_in_str(string):
    counter = 0
    for character in string:
        if character == ":":
            counter += 1
    if counter != 1:
        return False
    return True


# function in creating a dictionary / recalling from a dictionary.
def create_prefix_dict(s: Worksheet):
    # create an empty dict
    prefix_dict = {}

    # add prefix values according to the prefix sheet
    for col in s.iter_cols(max_col=1):
        for cell in col:
            row = cell.row
            if (
                cell.value is None
                or cell.value == "Prefix"
                or cell.value == "Prefix Sheet"
            ):
                pass
            else:
                # dynamically allocate the prefix sheet
                try:
                    prefix_dict[s[f"A{row}"].value] = s[f"B{row}"].value
                except Exception as e:
                    raise ConversionError(
                        f"Prefix processing error, sheet {s}, row {row}, error: {e}"
                    )
    return prefix_dict


# prefix and namespace use without list output
def using_prefix_and_namespace_non_list_output(cell_value, prefix, s: Worksheet, row):
    c = cell_value
    if c is not None:
        if c.startswith("http://") or c.startswith("https://"):
            pass
        elif only_one_colon_in_str(c):
            split_c_prefix = c.split(":")[0]
            split_c_added_component = c.split(":")[1]

            if split_c_prefix in prefix:
                c = prefix[split_c_prefix] + split_c_added_component
            else:
                print(
                    f"the prefix used: '{split_c_prefix}' in sheet {s} and row {row} "
                    f"isn't included in the prefix sheet"
                )
                raise Exception
        else:
            print(
                f"Something doesn't look right on row {row} and sheet {s}. "
                f"Likely this isn't in http form or more colons are used than normal"
            )
            raise Exception
    return c


# specifically just for concept scheme
def using_prefix_and_namespace_cs(cell_value, prefix):
    c = cell_value
    if c is not None:
        if c.startswith("http://") or c.startswith("https://"):
            pass
        elif only_one_colon_in_str(c):
            split_c_prefix = c.split(":")[0]
            split_c_added_component = c.split(":")[1]

            if split_c_prefix in prefix:
                c = prefix[split_c_prefix] + split_c_added_component
            else:
                print(
                    f"the prefix used: '{split_c_prefix}' in the concept scheme page"
                    f"isn't included in the prefix sheet"
                )
                raise Exception
        else:
            print(
                f"Something doesn't look right in the concept scheme page. "
                f"Likely the cells using prefixes aren't in http form or more colons are used than normal"
            )
            raise Exception
    return c


# prefix and namespace list with list output
def using_prefix_and_namespace(cell_value, prefix, s: Worksheet, row):
    variables = []
    for i in cell_value:
        if cell_value is not None:
            try:
                c = i
                if c.startswith("http://") or c.startswith("https://"):
                    pass
                elif only_one_colon_in_str(c):
                    split_c_prefix = c.split(":")[0]
                    split_c_added_component = c.split(":")[1]

                    if split_c_prefix in prefix:
                        c = prefix[split_c_prefix] + split_c_added_component
                    else:
                        print(
                            f"the prefix used: '{split_c_prefix}' in sheet {s} and row {row} "
                            f"isn't included in the prefix sheet"
                        )
                        raise Exception
                else:
                    print(
                        f"Something doesn't look right on row {row} and sheet {s}. "
                        f"Likely this isn't in http form or more colons are used than normal"
                    )
                    raise Exception
                variables.append(c)
            except Exception as e:
                logging.exception(e)
    return variables


# template version 0.4.3 using prefixes
def extract_concepts_and_collections(
    q: Worksheet, r: Worksheet, s: Worksheet, prefix
) -> Tuple[List[models.Concept], List[models.Collection]]:
    concepts = []
    collections = []
    # Iterating over the concept page and the additional concept page
    for col in q.iter_cols(max_col=1):
        for cell in col:
            row = cell.row
            if (
                cell.value is None
                or cell.value == "Concepts"
                or cell.value == "Concept IRI*"
            ):
                pass
            else:
                try:
                    c = models.Concept(
                        uri=using_prefix_and_namespace_non_list_output(
                            q[f"A{row}"].value, prefix, q, row
                        ),
                        pref_label=q[f"B{row}"].value,
                        pl_language_code=split_and_tidy_to_strings(q[f"C{row}"].value),
                        definition=q[f"D{row}"].value,
                        def_language_code=split_and_tidy_to_strings(q[f"E{row}"].value),
                        alt_labels=split_and_tidy_to_strings(q[f"F{row}"].value),
                        children=using_prefix_and_namespace(
                            split_and_tidy_to_strings(q[f"G{row}"].value),
                            prefix,
                            q,
                            row,
                        ),
                        provenance=q[f"H{row}"].value,
                        # Note in the new template, home_vocab_uri is synonymous with source vocab uri
                        home_vocab_uri=using_prefix_and_namespace_non_list_output(
                            q[f"I{row}"].value, prefix, q, row
                        ),
                        # additional concept features sheets
                        related_match=using_prefix_and_namespace(
                            split_and_tidy_to_strings(r[f"B{row}"].value),
                            prefix,
                            r,
                            row,
                        ),
                        close_match=using_prefix_and_namespace(
                            split_and_tidy_to_strings(r[f"C{row}"].value),
                            prefix,
                            r,
                            row,
                        ),
                        exact_match=using_prefix_and_namespace(
                            split_and_tidy_to_strings(r[f"D{row}"].value),
                            prefix,
                            r,
                            row,
                        ),
                        narrow_match=using_prefix_and_namespace(
                            split_and_tidy_to_strings(r[f"E{row}"].value),
                            prefix,
                            r,
                            row,
                        ),
                        broad_match=using_prefix_and_namespace(
                            split_and_tidy_to_strings(r[f"F{row}"].value),
                            prefix,
                            r,
                            row,
                        ),
                    )
                    concepts.append(c)
                except ValidationError as e:
                    raise ConversionError(
                        f"Concept processing error likely at sheet {q}, row {row}, and has error: {e}"
                    )

    # iterating over the collections page
    for col in s.iter_cols(max_col=1):
        for cell in col:
            row = cell.row
            if (
                cell.value is None
                or cell.value == "Collections"
                or cell.value == "Collection URI"
            ):
                pass
            else:
                try:
                    c = models.Collection(
                        uri=using_prefix_and_namespace_cs(s[f"A{row}"].value, prefix),
                        pref_label=s[f"B{row}"].value,
                        definition=s[f"C{row}"].value,
                        members=using_prefix_and_namespace(
                            split_and_tidy_to_strings(s[f"D{row}"].value),
                            prefix,
                            s,
                            row,
                        ),
                        provenance=s[f"E{row}"].value,
                    )
                    collections.append(c)
                except ValidationError as e:
                    raise ConversionError(
                        f"Collection processing error, likely at sheet {s}, row {row}, and has error: {e}"
                    )
    return concepts, collections


def extract_concept_scheme(sheet: Worksheet, prefix):
    cs = models.ConceptScheme(
        uri=using_prefix_and_namespace_cs(sheet["B2"].value, prefix),
        title=sheet["B3"].value,
        description=sheet["B4"].value,
        created=sheet["B5"].value,
        modified=sheet["B6"].value,
        creator=sheet["B7"].value,
        publisher=sheet["B8"].value,
        version=str(sheet["B9"].value),
        provenance=sheet["B10"].value,
        custodian=sheet["B11"].value,
        pid=sheet["B12"].value,
    )
    return cs



import datetime
from typing import List, Tuple

from openpyxl.worksheet.worksheet import Worksheet
from pydantic import ValidationError

from vocexcel import models
from vocexcel.utils import ConversionError, split_and_tidy_to_strings


# this is a new function to iterate over the collection sheet in template version 0.4.0
def extract_concepts_and_collections(
    q: Worksheet, r: Worksheet, s: Worksheet
) -> Tuple[List[models.Concept], List[models.Collection]]:
    concepts = []
    collections = []
    # Iterating over the concept page and the additional concept page
    for col in q.iter_cols(max_col=1):
        for cell in col:
            row = cell.row
            if (
                cell.value is None
                or cell.value == "Concepts"
                or cell.value == "Concept IRI*"
            ):
                pass
            else:
                try:
                    c = models.Concept(
                        uri=q[f"A{row}"].value,
                        pref_label=q[f"B{row}"].value,
                        pl_language_code=split_and_tidy_to_strings(q[f"C{row}"].value),
                        definition=q[f"D{row}"].value,
                        def_language_code=split_and_tidy_to_strings(q[f"E{row}"].value),
                        alt_labels=split_and_tidy_to_strings(q[f"F{row}"].value),
                        children=split_and_tidy_to_strings(q[f"G{row}"].value),
                        provenance=q[f"H{row}"].value,
                        home_vocab_uri=q[f"I{row}"].value,
                        # additional concept features sheets
                        related_match=split_and_tidy_to_strings(r[f"B{row}"].value),
                        close_match=split_and_tidy_to_strings(r[f"C{row}"].value),
                        exact_match=split_and_tidy_to_strings(r[f"D{row}"].value),
                        narrow_match=split_and_tidy_to_strings(r[f"E{row}"].value),
                        broad_match=split_and_tidy_to_strings(r[f"F{row}"].value),
                    )
                    concepts.append(c)
                except ValidationError as e:
                    raise ConversionError(
                        f"Concept processing error, likely at sheet {q}, row {row}, and with error: {e}"
                    )

    # iterating over the collections page
    for col in s.iter_cols(max_col=1):
        for cell in col:
            row = cell.row
            if (
                cell.value is None
                or cell.value == "Collections"
                or cell.value == "Collection URI"
            ):
                pass
            else:
                try:
                    c = models.Collection(
                        uri=s[f"A{row}"].value,
                        pref_label=s[f"B{row}"].value,
                        definition=s[f"C{row}"].value,
                        members=split_and_tidy_to_strings(s[f"D{row}"].value),
                        provenance=s[f"E{row}"].value,
                    )
                    collections.append(c)
                except ValidationError as e:
                    raise ConversionError(
                        f"Collection processing error, likely at sheet {s}, column {col}, row {row}, error: {e}"
                    )
    return concepts, collections


def extract_concept_scheme(sheet: Worksheet):
    cs = models.ConceptScheme(
        uri=sheet["B2"].value,
        title=sheet["B3"].value,
        description=sheet["B4"].value,
        created=sheet["B5"].value,
        modified=sheet["B6"].value,
        creator=sheet["B7"].value,
        publisher=sheet["B8"].value,
        version=str(sheet["B9"].value),
        provenance=sheet["B10"].value,
        custodian=sheet["B11"].value,
        pid=sheet["B12"].value,
    )
    return cs



import datetime
from itertools import chain
from typing import List, Union

from openpyxl import Workbook
from pydantic import BaseModel, field_validator
from rdflib import Graph, Literal, URIRef
from rdflib.namespace import DCAT, DCTERMS, OWL, RDF, RDFS, SKOS, XSD

try:
    from utils import all_strings_in_list_are_iris, string_is_http_iri
except ImportError:
    import sys

    sys.path.append("..")
    from .utils import all_strings_in_list_are_iris, string_is_http_iri

ORGANISATIONS = {
    "CGI": URIRef("https://linked.data.gov.au/org/cgi"),
    "GA": URIRef("https://linked.data.gov.au/org/ga"),
    "GGIC": URIRef("https://linked.data.gov.au/org/ggic"),
    "GSQ": URIRef("https://linked.data.gov.au/org/gsq"),
    "ICSM": URIRef("https://linked.data.gov.au/org/icsm"),
    "DES": URIRef("https://linked.data.gov.au/org/des"),
    "BITRE": URIRef("https://linked.data.gov.au/org/bitre"),
    "CASA": URIRef("https://linked.data.gov.au/org/casa"),
    "ATSB": URIRef("https://linked.data.gov.au/org/atsb"),
}

ORGANISATIONS_INVERSE = {uref: name for name, uref in ORGANISATIONS.items()}


class ConceptScheme(BaseModel):
    uri: str
    title: str
    description: str
    created: datetime.date
    modified: datetime.date | None = None
    creator: str
    publisher: str
    provenance: str
    version: str | None = None
    custodian: str | None = None
    pid: str | None = None

    @field_validator("creator")
    def publisher_must_be_from_list2(cls, v):
        if v not in ORGANISATIONS.keys():
            raise ValueError(
                f"Organisations must selected from the Organisations list: {', '.join(ORGANISATIONS)}"
            )
        return v

    @field_validator("publisher")
    def publisher_must_be_from_list(cls, v):
        if v not in ORGANISATIONS.keys():
            raise ValueError(
                f"Organisations must selected from the Organisations list: {', '.join(ORGANISATIONS)}"
            )
        return v

    def to_graph(self):
        g = Graph()
        v = URIRef(self.uri)
        # For dcterms:identifier
        if "#" in v:
            identifier = v.split("#")[-1]
        else:
            identifier = v.split("/")[-1]
        g.add((v, DCTERMS.identifier, Literal(identifier, datatype=XSD.token)))

        g.add((v, RDF.type, SKOS.ConceptScheme))
        g.add((v, SKOS.prefLabel, Literal(self.title, lang="en")))
        g.add((v, SKOS.definition, Literal(self.description, lang="en")))
        g.add((v, DCTERMS.created, Literal(self.created, datatype=XSD.date)))
        if self.modified is not None:
            g.add((v, DCTERMS.modified, Literal(self.created, datatype=XSD.date)))
        else:
            g.add(
                (
                    v,
                    DCTERMS.modified,
                    Literal(
                        datetime.datetime.now().strftime("%Y-%m-%d"), datatype=XSD.date
                    ),
                )
            )
        g.add((v, DCTERMS.creator, ORGANISATIONS[self.creator]))
        g.add((v, DCTERMS.publisher, ORGANISATIONS[self.publisher]))
        if self.version is not None and self.version != "None":
            g.add((v, OWL.versionInfo, Literal(self.version)))
        g.add((v, DCTERMS.provenance, Literal(self.provenance, lang="en")))
        if self.custodian is not None:
            g.add((v, DCAT.contactPoint, Literal(self.custodian)))
        if self.pid is not None:
            # adding to the graph depending on if the pid is a URI or a literal
            if string_is_http_iri(self.pid)[0]:
                g.add((v, RDFS.seeAlso, URIRef(self.pid)))
            else:
                g.add((v, RDFS.seeAlso, Literal(self.pid)))

        # bind non-core prefixes
        g.bind("cs", v)
        g.bind(
            "",
            str(v).split("#")[0] if "#" in str(v) else "/".join(str(v).split("/")[:-1]),
        )
        g.bind("dcat", DCAT)
        g.bind("dcterms", DCTERMS)
        g.bind("skos", SKOS)
        g.bind("owl", OWL)

        return g

    def to_excel(self, wb: Workbook):
        ws = wb["Concept Scheme"]
        ws["B2"] = self.uri
        ws["B3"] = self.title
        ws["B4"] = self.description
        ws["B5"] = self.created.isoformat()
        ws["B6"] = self.modified.isoformat()
        ws["B7"] = self.creator
        ws["B8"] = self.publisher
        ws["B9"] = self.version
        ws["B10"] = self.provenance
        ws["B11"] = self.custodian
        ws["B12"] = self.pid


class Concept(BaseModel):
    uri: str
    pref_label: Union[str, List[str]]
    alt_labels: List[str] = []
    pl_language_code: List[str] = []
    definition: Union[str, List[str]]
    def_language_code: List[str] = []
    children: List[str] = []
    other_ids: List[str] = []
    home_vocab_uri: str | None = None
    provenance: str | None = None
    related_match: List[str] = []
    close_match: List[str] = []
    exact_match: List[str] = []
    narrow_match: List[str] = []
    broad_match: List[str] = []

    @field_validator("children")
    def each_c_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    @field_validator("related_match")
    def each_rm_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    @field_validator("close_match")
    def each_cm_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    @field_validator("exact_match")
    def each_em_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    @field_validator("narrow_match")
    def each_nm_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    @field_validator("broad_match")
    def each_bm_must_be_an_iri(cls, elem):
        r = all_strings_in_list_are_iris(elem)
        assert r[0], r[1]
        return elem

    def to_graph(self):
        g = Graph()
        c = URIRef(self.uri)

        g.add((c, RDF.type, SKOS.Concept))
        # For dcterms:identifier
        if "#" in c:
            identifier = c.split("#")[-1]
        else:
            identifier = c.split("/")[-1]
        g.add((c, DCTERMS.identifier, Literal(identifier, datatype=XSD.token)))

        if not self.pl_language_code:
            g.add((c, SKOS.prefLabel, Literal(self.pref_label, lang="en")))
        else:
            for lang_code in self.pl_language_code:
                g.add((c, SKOS.prefLabel, Literal(self.pref_label, lang=lang_code)))
        if self.alt_labels is not None:
            for alt_label in self.alt_labels:
                g.add((c, SKOS.altLabel, Literal(alt_label, lang="en")))
        if not self.def_language_code:
            g.add((c, SKOS.definition, Literal(self.definition, lang="en")))
        else:
            for lang_code in self.def_language_code:
                g.add((c, SKOS.definition, Literal(self.definition, lang=lang_code)))
        for child in self.children:
            g.add((c, SKOS.narrower, URIRef(child)))
            g.add((URIRef(child), SKOS.broader, c))
        if self.other_ids is not None:
            for other_id in self.other_ids:
                g.add((c, SKOS.notation, Literal(other_id)))
        if self.home_vocab_uri is not None:
            g.add((c, RDFS.isDefinedBy, URIRef(self.home_vocab_uri)))
        if self.provenance is not None:
            g.add((c, DCTERMS.provenance, Literal(self.provenance, lang="en")))
        if self.related_match is not None:
            for related_match in self.related_match:
                g.add((c, SKOS.relatedMatch, URIRef(related_match)))
        if self.close_match:
            for close_match in self.close_match:
                g.add((c, SKOS.closeMatch, URIRef(close_match)))
        if self.exact_match is not None:
            for exact_match in self.exact_match:
                g.add((c, SKOS.exactMatch, URIRef(exact_match)))
        if self.narrow_match is not None:
            for narrow_match in self.narrow_match:
                g.add((c, SKOS.narrowMatch, URIRef(narrow_match)))
        if self.broad_match is not None:
            for broad_match in self.broad_match:
                g.add((c, SKOS.broadMatch, URIRef(broad_match)))

        return g

    def to_excel(self, wb: Workbook, row_no_features: int, row_no_concepts: int):
        """ "
        Export Concept to Excel using on row per language

        Non-labels like Children, Provenance are reported only for the first
        language. If "en" is among the used languages, it is reported first.
        """
        # Note: "self.other_ids" is no longer supported in template 0.4.3

        ws = wb["Concepts"]

        # determine the languages with full and patial translation
        pref_labels = {
            lang: pl for pl, lang in zip(self.pref_label, self.pl_language_code)
        }
        definitions = {
            lang: d for d, lang in zip(self.definition, self.def_language_code)
        }
        fully_translated = [l for l in pref_labels.keys() if l in definitions.keys()]
        partially_translated = [
            l
            for l in chain(pref_labels.keys(), definitions.keys())
            if l not in fully_translated
        ]

        # put "en" first if available
        if "en" in fully_translated:
            fully_translated.remove("en")
            fully_translated.insert(0, "en")

        first_row_exported = False
        for lang in chain(fully_translated, partially_translated):
            ws[f"A{row_no_concepts}"] = self.uri
            ws[f"B{row_no_concepts}"] = pref_labels.get(lang, "")
            ws[f"C{row_no_concepts}"] = lang
            ws[f"D{row_no_concepts}"] = definitions.get(lang, "")
            ws[f"E{row_no_concepts}"] = lang
            ws[f"H{row_no_concepts}"] = self.provenance

            if first_row_exported:
                row_no_concepts += 1
                continue
            else:
                first_row_exported = True
            ws[f"F{row_no_concepts}"] = ",\n".join(self.alt_labels)
            ws[f"G{row_no_concepts}"] = ",\n".join(self.children)
            ws[f"I{row_no_concepts}"] = self.home_vocab_uri
            row_no_concepts += 1

        ws = wb["Additional Concept Features"]

        ws[f"A{row_no_features}"] = self.uri
        ws[f"B{row_no_features}"] = ",\n".join(self.related_match)
        ws[f"C{row_no_features}"] = ",\n".join(self.close_match)
        ws[f"D{row_no_features}"] = ",\n".join(self.exact_match)
        ws[f"E{row_no_features}"] = ",\n".join(self.narrow_match)
        ws[f"F{row_no_features}"] = ",\n".join(self.broad_match)

        return row_no_concepts


class Collection(BaseModel):
    uri: str
    pref_label: str
    definition: str
    members: List[str]
    provenance: str | None = None

    @field_validator("members")
    def members_must_by_iris(cls, v):
        if any([not i.startswith("http") for i in v]):
            raise ValueError("The members of a Collection must be a list of IRIs")
        return v

    def to_graph(self):
        g = Graph()
        c = URIRef(self.uri)
        g.add((c, RDF.type, SKOS.Collection))
        # for dcterms:identifier
        if "#" in c:
            identifier = c.split("#")[-1]
        else:
            identifier = c.split("/")[-1]
        g.add((c, DCTERMS.identifier, Literal(identifier, datatype=XSD.token)))

        g.add((c, SKOS.prefLabel, Literal(self.pref_label, lang="en")))
        g.add((c, SKOS.definition, Literal(self.definition, lang="en")))
        for member in self.members:
            g.add((c, SKOS.member, URIRef(member)))
        if self.provenance is not None:
            g.add((c, DCTERMS.provenance, Literal(self.provenance, lang="en")))

        return g

    def to_excel(self, wb: Workbook, row_no: int):
        ws = wb["Collections"]
        ws[f"A{row_no}"] = self.uri
        ws[f"B{row_no}"] = self.pref_label
        ws[f"C{row_no}"] = self.definition
        ws[f"D{row_no}"] = ",\n".join(self.members)
        ws[f"E{row_no}"] = self.provenance


class Vocabulary(BaseModel):
    concept_scheme: ConceptScheme
    concepts: List[Concept]
    collections: List[Collection]

    def to_graph(self):
        g = self.concept_scheme.to_graph()
        cs = URIRef(self.concept_scheme.uri)
        for concept in self.concepts:
            g += concept.to_graph()
            g.add((URIRef(concept.uri), SKOS.inScheme, cs))
        for collection in self.collections:
            g += collection.to_graph()
            g.add((URIRef(collection.uri), DCTERMS.isPartOf, cs))
            g.add((cs, DCTERMS.hasPart, URIRef(collection.uri)))

        # create as Top Concepts those Concepts that have no skos:narrower properties with them as objects
        for s in g.subjects(SKOS.inScheme, cs):
            is_tc = True
            for _ in g.objects(s, SKOS.broader):
                is_tc = False
            if is_tc:
                g.add((cs, SKOS.hasTopConcept, s))
                g.add((s, SKOS.topConceptOf, cs))

        return g



import sys
from pathlib import Path

import pytest

sys.path.append(str(Path(__file__).parent.parent.absolute() / "vocexcel"))
from pydantic.error_wrappers import ValidationError

from vocexcel.models import *


def test_vocabulary_valid():
    v = ConceptScheme(
        uri="https://linked.data.gov.au/def/borehole-start-point",
        title="Borehole Start Point",
        description="Indicates the nature of the borehole start point location",
        created="2020-04-02",
        modified="2020-04-02",
        creator="GSQ",
        publisher="GSQ",
        version="",
        provenance="Derived from the 2011-09 version of CGI Borehole start point list",
        custodian="Vance Kelly",
        pid="http://pid.geoscience.gov.au/dataset/ga/114541",
    )


def test_vocabulary_invalid_uri():
    with pytest.raises(ValidationError):
        v = ConceptScheme(
            uri="ftp://linked.data.gov.au/def/borehole-start-point",
            title="Borehole Start Point",
            description="Indicates the nature of the borehole start point location",
            created="2020-04-02",
            modified="02/042020",
            creator="GSQ",
            publisher="GSQ",
            version="",
            provenance="Derived from the 2011-09 version of CGI Borehole start point list",
            custodian="Vance Kelly",
            pid="http://pid.geoscience.gov.au/dataset/ga/114541",
        )


def test_vocabulary_invalid_created_date():
    with pytest.raises(ValidationError):
        v = ConceptScheme(
            uri="https://linked.data.gov.au/def/borehole-start-point",
            title="Borehole Start Point",
            description="Indicates the nature of the borehole start point location",
            created="2020-04",
            modified="2020-04-02",
            creator="GSQ",
            publisher="GSQ",
            version="",
            provenance="Derived from the 2011-09 version of CGI Borehole start point list",
            custodian="Vance Kelly",
            pid="http://pid.geoscience.gov.au/dataset/ga/114541",
        )


def test_concept():
    # uri
    # pref_label
    # alt_labels
    # pl_language_code
    # definition
    # def_language_code
    # children
    # other_ids
    # home_vocab_uri
    # provenance
    c = Concept(
        uri="https://example.com/thing/x",
        pref_label="Thing X",
        definition="Fake def for Thing X",
        children=["https://example.com/thing/y", "https://example.com/thing/z"],
        other_ids=["XX", "XXX"],
        close_match=[
            "https://example.com/thing/other",
            "https://example.com/thing/otherother",
        ],
    )
    actual = c.to_graph()
    expected = Graph().parse(
        data="""@prefix skos: <http://www.w3.org/2004/02/skos/core#> .
        @prefix dcterms: <http://purl.org/dc/terms/> .
        @prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<https://example.com/thing/y> skos:broader <https://example.com/thing/x> .
<https://example.com/thing/z> skos:broader <https://example.com/thing/x> .
<https://example.com/thing/x> a skos:Concept ;
    skos:closeMatch <https://example.com/thing/other>, <https://example.com/thing/otherother> ;
    skos:definition "Fake def for Thing X"@en ;
    skos:narrower <https://example.com/thing/y>, <https://example.com/thing/z> ;
    skos:notation "XX", "XXX" ;
    dcterms:identifier "x"^^xsd:token ;
    skos:prefLabel "Thing X"@en ."""
    )
    assert actual.isomorphic(expected)


def test_concept_iri():
    # this is testing that children list elements are IRIs, not just ordinary strings
    # uri
    # pref_label
    # alt_labels
    # pl_language_code
    # definition
    # def_language_code
    # children
    # other_ids
    # home_vocab_uri
    # provenance
    with pytest.raises(ValidationError) as e:
        c = Concept(
            uri="https://example.com/thing/x",
            pref_label="Thing X",
            definition="Fake def for Thing X",
            children=["broken iri", "http://example.com/working-iri"],  # non-IRI string
        )

    with pytest.raises(ValidationError) as e:
        c = Concept(
            uri="https://example.com/thing/x",
            pref_label="Thing X",
            definition="Fake def for Thing X",
            children=[
                "ftp://example.com/working-iri",
                "http://example.com/working-iri",
            ],  # IRI starts ftp
        )

    # valid children, invalid related_match
    with pytest.raises(ValidationError) as e:
        c = Concept(
            uri="https://example.com/thing/x",
            pref_label="Thing X",
            definition="Fake def for Thing X",
            related_match=[
                "http://example.com/working-iri/rm/1",
                "http://example.com/working-iri/rm/x",  # no x
                "ftp://example.com/working-iri/rm/3",
            ],
            children=[
                "http://example.com/working-iri/c/1",
                "http://example.com/working-iri/c/2",
            ],
        )
